@@ -34,15 +34,23 @@
 #include "jtaglib.h"
 #include "ctrlc.h"
 
+struct pif_device {
+  struct device		base;
+  struct jtdev		jtag;
+};
+
 /*============================================================================*/
 /* pif MSP430 JTAG operations */
 
 /*----------------------------------------------------------------------------*/
-/* Read a word-aligned block from any kind of memory */
-static int read_words( struct jtdev *p, address_t addr,
-		       address_t len,
-		       uint8_t*  data )
+/* Read a word-aligned block from any kind of memory
+ * returns the number of bytes read or -1 on failure
+ */
+static int read_words(device_t dev, const struct chipinfo_memory *m,
+		address_t addr, address_t len, uint8_t *data)
 {
+  struct pif_device *pif = (struct pif_device *)dev;
+  struct jtdev *p = &pif->jtag;
   unsigned int index;
   unsigned int word;
 
@@ -52,18 +60,15 @@ static int read_words( struct jtdev *p, address_t addr,
     data[index+1] = (word >> 8) & 0x00ff;
   }
 
-  return p->failed ? -1 : 0;
+  return p->failed ? -1 : len;
 }
 
 /*----------------------------------------------------------------------------*/
 /* Write a word to RAM */
-int write_ram_word( struct jtdev *p, address_t addr,
+static int write_ram_word( struct jtdev *p, address_t addr,
 		    uint16_t  value )
 {
-  unsigned int word;
-
-  word = ((value & 0x00ff) << 8) | ((value & 0xff00) >> 8);
-  jtag_write_mem( p, 16, addr, word );
+  jtag_write_mem( p, 16, addr, value );
 
   return p->failed ? -1 : 0;
 }
@@ -95,36 +100,35 @@ static int write_flash_block( struct jtdev *p, address_t addr,
   return p->failed ? -1 : 0;
 }
 
-/*----------------------------------------------------------------------------*/
-/* Write a single byte by reading and rewriting a word. */
-static int write_byte( struct jtdev *p,
-		       address_t addr,
-		       uint8_t   value )
+/* Write a word-aligned block to any kind of memory.
+ * returns the number of bytes written or -1 on failure
+ */
+static int write_words(device_t dev, const struct chipinfo_memory *m,
+		address_t addr, address_t len, const uint8_t *data)
 {
-  address_t aligned = addr & ~1;
-  uint8_t data[2];
-  unsigned int word;
+  struct pif_device *pif = (struct pif_device *)dev;
+  struct jtdev *p = &pif->jtag;
+  int r;
 
-  read_words(p, aligned, 2, data);
-  data[addr & 1] = value;
-
-  if ( ADDR_IN_FLASH(addr) ) {
-    /* program in FLASH */
-    write_flash_block(p, aligned, 2, data);
+  if (m->type != CHIPINFO_MEMTYPE_FLASH) {
+    len = 2;
+    r = write_ram_word(p, addr, r16le(data));
   } else {
-    /* write to RAM */
-    word = (uint16_t)data[1] | ((uint16_t)data[0] << 8);
-    write_ram_word(p, aligned, word);
+    r = write_flash_block(p, addr, len, data);
   }
 
-  return p->failed ? -1 : 0;
+  if (r < 0) {
+    printc_err("pif: write_words at address 0x%x failed\n", addr);
+    return -1;
+  }
+
+  return len;
 }
 
 /*----------------------------------------------------------------------------*/
 static int init_device(struct jtdev *p)
 {
   unsigned int jtag_id;
-  unsigned int chip_id;
 
   printc_dbg("Starting JTAG\n");
   jtag_id = jtag_init(p);
@@ -135,19 +139,11 @@ static int init_device(struct jtdev *p)
     return -1;
   }
 
-  chip_id =jtag_chip_id(p);
-  printc_dbg("Chip ID: %04X\n", chip_id);
-
   return 0;
 }
 
 /*===== MSPDebug Device interface ============================================*/
 
-struct pif_device {
-  struct device		base;
-  struct jtdev		jtag;
-};
-
 /*----------------------------------------------------------------------------*/
 static int refresh_bps(struct pif_device *dev)
 {
@@ -192,38 +188,8 @@ static int pif_readmem( device_t  dev_base,
 			address_t len )
 {
   struct pif_device *dev = (struct pif_device *)dev_base;
-  uint8_t  data[2];
-
   dev->jtag.failed = 0;
-
-  if ( len > 0 ) {
-    /* Handle unaligned start */
-    if (addr & 1) {
-      if (read_words(&dev->jtag, addr & ~1, 2, data) < 0)
-	return -1;
-      mem[0] = data[1];
-      addr++;
-      mem++;
-      len--;
-    }
-
-    /* Read aligned blocks */
-    if (len >= 2) {
-      if (read_words(&dev->jtag, addr, len & ~1, mem) < 0)
-	return -1;
-      addr += len & ~1;
-      mem += len & ~1;
-      len &= 1;
-    }
-
-    /* Handle unaligned end */
-    if (len == 1) {
-      if (read_words(&dev->jtag, addr, 2, data) < 0)
-	return -1;
-      mem[0] = data[0];
-    }
-  }
-  return 0;
+  return readmem(dev_base, addr, mem, len, read_words);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -233,45 +199,8 @@ static int pif_writemem( device_t       dev_base,
 			 address_t      len )
 {
   struct pif_device *dev = (struct pif_device *)dev_base;
-
   dev->jtag.failed = 0;
-
-  if (len > 0)
-  {
-    /* Handle unaligned start */
-    if (addr & 1) {
-      if (write_byte(&dev->jtag, addr, mem[0]) < 0)
-	return -1;
-      addr++;
-      mem++;
-      len--;
-    }
-
-    /* Write aligned blocks */
-    while (len >= 2) {
-      if ( ADDR_IN_FLASH(addr) ) {
-	if (write_flash_block(&dev->jtag, addr, len & ~1, mem) < 0)
-	  return -1;
-	addr += len & ~1;
-	mem += len & ~1;
-	len &= 1;
-      } else {
-	if (write_ram_word(&dev->jtag, addr,
-		(uint16_t)mem[1] | ((uint16_t)mem[0] << 8)) < 0)
-	  return -1;
-	addr += 2;
-	mem  += 2;
-	len  -= 2;
-      }
-    }
-
-    /* Handle unaligned end */
-    if (len == 1) {
-      if (write_byte(&dev->jtag, addr, mem[0]) < 0)
-	return -1;
-    }
-  }
-  return 0;
+  return writemem(dev_base, addr, mem, len, write_words, read_words);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -383,6 +312,14 @@ static int pif_erase( device_t dev_base,
   return dev->jtag.failed ? -1 : 0;
 }
 
+/*----------------------------------------------------------------------------*/
+static int pif_getconfigfuses(device_t dev_base)
+{
+  struct pif_device *dev = (struct pif_device *)dev_base;
+
+  return jtag_get_config_fuses(&dev->jtag);
+}
+
 /*----------------------------------------------------------------------------*/
 
 
@@ -409,6 +346,7 @@ static device_t pif_open(const struct device_args *args)
   memset(dev, 0, sizeof(*dev));
   dev->base.type = &device_pif;
   dev->base.max_breakpoints = 2; //supported by all devices
+  dev->base.need_probe = 1;
   (&dev->jtag)->f = &jtdev_func_pif;
 
   if ((&dev->jtag)->f->jtdev_open(&dev->jtag, args->path) < 0) {
@@ -469,6 +407,50 @@ static device_t gpio_open(const struct device_args *args)
   return &dev->base;
 }
 
+/*----------------------------------------------------------------------------*/
+
+
+static device_t bp_open(const struct device_args *args)
+{
+  struct pif_device *dev;
+
+  if (!(args->flags & DEVICE_FLAG_TTY)) {
+    printc_err("bp: this driver does not support raw USB access\n");
+    return NULL;
+  }
+
+  if (!(args->flags & DEVICE_FLAG_JTAG)) {
+    printc_err("bp: this driver does not support Spy-Bi-Wire\n");
+    return NULL;
+  }
+
+  dev = malloc(sizeof(*dev));
+  if (!dev) {
+    printc_err("bp: malloc: %s\n", last_error());
+    return NULL;
+  }
+
+  memset(dev, 0, sizeof(*dev));
+  dev->base.type = &device_pif;
+  dev->base.max_breakpoints = 2; //supported by all devices
+  dev->base.need_probe = 1;
+  (&dev->jtag)->f = &jtdev_func_bp;
+
+  if ((&dev->jtag)->f->jtdev_open(&dev->jtag, args->path) < 0) {
+    printc_err("bp: can't open port\n");
+    free(dev);
+    return NULL;
+  }
+
+  if (init_device(&dev->jtag) < 0) {
+    printc_err("bp: initialization failed\n");
+    free(dev);
+    return NULL;
+  }
+
+  return &dev->base;
+}
+
 /*----------------------------------------------------------------------------*/
 static void pif_destroy(device_t dev_base)
 {
@@ -493,7 +475,8 @@ const struct device_class device_pif = {
   .setregs  = pif_setregs,
   .ctl      = pif_ctl,
   .poll     = pif_poll,
-  .erase    = pif_erase
+  .erase    = pif_erase,
+  .getconfigfuses = pif_getconfigfuses
 };
 
 const struct device_class device_gpio = {
@@ -507,5 +490,21 @@ const struct device_class device_gpio = {
   .setregs  = pif_setregs,
   .ctl      = pif_ctl,
   .poll     = pif_poll,
-  .erase    = pif_erase
+  .erase    = pif_erase,
+  .getconfigfuses = pif_getconfigfuses
+};
+
+const struct device_class device_bp = {
+  .name     = "bus-pirate",
+  .help     = "Bus Pirate JTAG, MISO-TDO, MOSI-TDI, CS-TMS, AUX-RESET, CLK-TCK",
+  .open     = bp_open,
+  .destroy  = pif_destroy,
+  .readmem  = pif_readmem,
+  .writemem = pif_writemem,
+  .getregs  = pif_getregs,
+  .setregs  = pif_setregs,
+  .ctl      = pif_ctl,
+  .poll     = pif_poll,
+  .erase    = pif_erase,
+  .getconfigfuses = pif_getconfigfuses
 };
