@@ -28,11 +28,10 @@
 
 #include "device.h"
 
-#define ADDR_IN_FLASH(a)        ( (a) >= 0x1000 )
-
 /* pif implementation */
 extern const struct device_class device_pif;
 /* share wiht gpio implementation */
 extern const struct device_class device_gpio;
+extern const struct device_class device_bp;
 
 #endif
