@@ -17,7 +17,19 @@
  */
 
 #include "input.h"
+
+#ifdef USE_READLINE
+
+#include "input_readline.h"
+
+/* Default input module */
+const struct input_interface *input_module = &input_readline;
+
+#else
+
 #include "input_console.h"
 
 /* Default input module */
 const struct input_interface *input_module = &input_console;
+
+#endif
