@@ -24,5 +24,8 @@ int cmd_help(char **arg);
 int cmd_read(char **arg);
 int cmd_opt(char **arg);
 int cmd_exit(char **arg);
+#ifndef NO_SHELLCMD
+int cmd_shellcmd(char **arg);
+#endif
 
 #endif
