@@ -21,17 +21,13 @@
 
 /* Portable dynamic loader interface. */
 
-#ifdef WIN32
+#ifdef __Windows__
 #include <windows.h>
 
 typedef HMODULE dynload_handle_t;
-#else /* WIN32 */
+#else /* __Windows__ */
 typedef void *dynload_handle_t;
-#endif
-
-#ifndef WINAPI
-#define WINAPI
-#endif
+#endif /* __Windows__ */
 
 dynload_handle_t dynload_open(const char *filename);
 void dynload_close(dynload_handle_t hnd);
